'
'  pb_file.bas
'
'  PB-32bit misc file routines
'
'  Function FileExists(sFile As String) As Integer
'  Function ExtractPath (zFullFile As String) As String
'  Function ExtractFileName(zFillFile as String) as String
'  Function GetExeName() As String
'  Function GetModuleName(ByVal sModule as String) as String
'  Function GetTempFile(seed$, ext$) As String
'  Function GetFileDateTime(sFileName as String) as String
'  Function GetFileSize(sFileName as String) as String
'  Function GetWindowsDir() as String
'  Function GetSystemDir() as String
'  Function GetTemporaryDir() as String
'  Sub DeleteFileSpec(ByVal sFileSpec As String)
'  Sub EnsureDirectoryExists(ByVal sFileName As String)
'
'  DEPENDS UPON
'     win32api.inc
'

#if not %def(%PB_FILE_INC)
%PB_FILE_INC = 1

'============================================================================
Function ExtractFileName(ByVal sFileName As String) As String

   Function = Parse$(sFileName, Any "\/:", -1)

End Function

'============================================================================
Function FileExists(ByVal sFile As String) As Integer

   If Dir$(sFile) = "" Then
      Function = %FALSE
   Else
      Function = %TRUE
   End If

End Function

'
'  EnsureDirectoryExists
'  Creates all subdirectories if they don't already exist.
'  You can specify a full file name and or just the path.
'  If just the path is specified, it must end in a backslash
'============================================================================
Sub EnsureDirectoryExists(ByVal sFileName As String)

   Dim i As Long
   Dim sDrive As String

   If Instr(sFileName, "\") < 1 Then Exit Sub

   '- All errors are ignored
   On Error Resume Next
   sDrive = ""
   For i = 1 To ParseCount(sFileName, "\")
      If sDrive = "" Then
         sDrive = Parse$(sFileName, "\", 1)
      Else
         sDrive = sDrive + "\" + Parse$(sFileName, "\", i)
      End If
      MkDir sDrive
   Next i

   ErrClear
   On Error GoTo 0

End Sub

'============================================================================
Function ExtractPath (zFullFile As String) As String

   '- This routine strips the file
   '  path from the full name.
   '
   Dim i As Integer
   Dim iStartAccum As Integer
   Dim iLen As Integer
   Dim sFile As String
   Dim sAccum As String

   sFile = Ltrim$(Rtrim$(zFullFile))
   iLen = Len(sFile)
   sAccum = ""

   iStartAccum = %False
   If iLen > 0 Then
      For i = iLen To 1 Step -1
         If (Mid$(sFile, i, 1) = "\") Or _
            (Mid$(sFile, i, 1) = ":") Then
            iStartAccum = %True
         End If
         If iStartAccum = %True Then
            sAccum = Mid$(sFile, i, 1) + sAccum
         End If
      Next i
   End If

   Function = sAccum

End Function

'============================================================================
Function GetModuleName(ByVal sModule as String) as String
   Dim zFileName As Asciiz * 400

   GetModuleFileName GetModuleHandle(ByCopy sModule + $NUL), zFileName, 400
   Function = Trim$(zFileName)

End Function

'============================================================================
Function GetExeName() As String

   Dim zFileName As Asciiz * 400

   GetModuleFileName ByVal %NULL, zFileName, 400
   Function = Trim$(zFileName)

End Function

'============================================================================
Function GetTempFile(seed$, ext$) As String
'
'  Takes a seed value (up to 4 characters)
'  and an extension (e.g. ".ddc") and
'  returns a unique file name.
'

   Dim e%
   Dim iCount As Long
   Dim sName As String
   Dim sPath As String
   Dim sTest As String
   Dim sFormat As String
   Dim zPath As Asciiz * 300
   Dim zSeed As Asciiz * 100

   '- Get Temp Path
   GetTempPath SizeOf(zPath), zPath

   sPath = Trim$(zPath)
   If Right$(sPath$, 1) <> "\" Then
      sPath$ = sPath$ + "\"
   End If

   If Len(seed$) > 7 Then seed$ = Left$(seed$,7)
   sFormat = String$(8 - Len(seed$), "0")
   sName = ""
   For iCount = 1 To 9999
      sTest = sPath + seed$ + Format$(iCount, sFormat) + ext$
      If FileExists(sTest) = %False Then
         sName = sTest
         Exit For
      End If
   Next iCount

   Function = sName

End Function

'============================================================================
Function GetTemporaryDir() Export As String

   Dim zPath As Asciiz * 500
   Dim sPath As String

   GetTempPath SizeOf(zPath), zPath
   sPath = Trim$(zPath)
   If Right$(sPath, 1) <> "\" Then sPath = sPath + "\"
   Function = sPath

End Function

'============================================================================
Function GetWindowsDir() Export As String

   Dim zPath As Asciiz * 500
   Dim sPath As String

   GetWindowsDirectory zPath, SizeOf(zPath)
   sPath = Trim$(zPath)
   If Right$(sPath, 1) <> "\" Then sPath = sPath + "\"
   Function = sPath

End Function

'============================================================================
Function GetSystemDir() Export As String

   Dim zPath As Asciiz * 500
   Dim sPath As String

   GetSystemDirectory zPath, SizeOf(zPath)
   sPath = Trim$(zPath)
   If Right$(sPath, 1) <> "\" Then sPath = sPath + "\"
   Function = sPath

End Function

'============================================================================
Sub DeleteFileSpec(ByVal sFileSpec As String)

   Dim i As Long
   Dim iCount As Long
   Dim sF As String
   Dim sPath As String
   Dim sFiles() As String

   On Error Resume Next

   ReDim sFiles(0 To 0) As String
   iCount = 0

   sPath = Trim$(ExtractPath(sFileSpec))
   If Right$(sPath, 1) <> "\" Then
      sPath = sPath + "\"
   End If

   sF = Dir$(sFileSpec)
   Do Until sF = ""
      iCount = iCount + 1
      ReDim Preserve sFiles(0 To iCount) As String
      sFiles(iCount) = sPath + sF
      sF = Dir$
   Loop
   For i = 1 To iCount
      errclear
      Kill sFiles(i)
   Next i

End Sub

#endif
