'
'  rserver.bas
'
'  multi-threaded, multi-homed, tcp-based, remote binary data server
'
'  By Don Dickinson
'  ddickinson@usinternet.com
'  dickinson.basicguru.com
'
'  Hereby Public Domain
'  Use implies that you hold the author, Don Dickinson, harmless from any
'  responsibility for effects or side-effects of using this code. It is 
'  provided in good faith by the auther - you are using it at your own
'  risk.
'

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Compiler Directives
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Compile Exe
#Dim All

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Included files
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Include "win32api.inc"          'windows api, constants, structures
#Include "wsconst.inc"           'winsock structures and constants
#Include "pb_file.bas"           'various file and directory routines
#Include "pb_ini.bas"            '.ini file handling
#Include "rserver.inc"           'rserver specific constants and structures

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Constants
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%LOG_DETAILED = %False

'- Main Screen
%labIP      = 801
%labPort    = 802
%labDate    = 803
%labHits    = 804
%labActive  = 805
%labAction  = 806
%btnSetup   = 807
%timer1     = 808

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Globals
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Global gDlgMain As Long          'handle for main dialog window
Global gLog  As Long             'handle to log file
Global gShutdown as Long         'flag telling system to abort and close conns
Global gSock() As Long           'array of client connection sockets
Global gMainSock() as Long       'array of listening sockets
Global gThread() as Long         'array of running threads
Global gThreadRunning() as Long  'array that indicates if a thread has finished
Global gActive As Long           'number of active connections
Global gHits As Long             'total hits
Global gPort As Long             'connection port
Global gIPAddr() as Long         'array of ip addresses being listed to
Global gIPCount as Long          'count of ip addresses being listed to
Global gCS As CRITICAL_SECTION   'critical section structure
Global gIniFile as String        'ini file name and path

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  tcpSafeReceive
'  Specify a length of data to wait for and this function
'  retrieves data until all data has filled the buffer or
'  an error occurs.
'  Returns: %True on success, %False on error
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function tcpSafeReceive(ByVal hSocket As Long, ByVal iBufferLen As Long, _
                        recBuff As String) As Long

   Dim iLeft As Long
   Dim sBuffer As String

   recBuff = ""
   iLeft = iBufferLen

   Do
      sBuffer = Space$(iLeft)

      On Error Resume Next
      sBuffer = Space$(iBufferLen)
      Tcp Recv hSocket, iLeft, sBuffer
      If Err Then
         Function = %False
         Exit Function
      End If

      recBuff = recBuff + sBuffer
      If Len(recBuff) >= iBufferLen Then
         Exit Do
      End If
      iLeft = iBufferLen - Len(recBuff)
      Sleep 5
   Loop
   Function = %True

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  FormatIP
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function FormatIP(RawAddr As Long) As String

   Dim sAddr As String

   sAddr = Mkl$(RawAddr)
   Function = Format$(Asc(Mid$(sAddr, 1, 1))) + "." + _
              Format$(Asc(Mid$(sAddr, 2, 1))) + "." + _
              Format$(Asc(Mid$(sAddr, 3, 1))) + "." + _
              Format$(Asc(Mid$(sAddr, 4, 1)))
End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  LogEvent
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub LogEvent(ByVal buffer As String)

   if gLog then 
      Print #gLog, Date$ + " " + Time$ + " " + Buffer
   end if

End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  EnterCritical
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub EnterCritical
   EnterCriticalSection gCS
End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  LeaveCritical
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub LeaveCritical
   LeaveCriticalSection gCS
End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  DisplayEvent
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub DisplayEvent(ByVal buffer As String)

   gHits = gHits + 1
   If gHits > 2000000000 Then
      gHits = 1
   End If

   Control Set Text gDlgMain, %labAction, Time$ + "  " + Buffer
   Control Set Text gDlgMain, %labActive, Format$(gActive)
   Control Set Text gDlgMain, %labHits, Format$(gHits)

End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  ThreadCleanup
'  If a thread has finished executing, but the thread handle
'  has not been closed, I close it here. This is called by a 
'  timer that goes off every %TIMER_INTERVAL milliseconds
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub ThreadCleanup()
   
   Dim i as Long
   Dim foo as Long
   
   For i = 1 to %MAX_CONNECTIONS
      if gThread(i) then
         if gThreadRunning(i) = %False then
            Thread Close gThread(i) to foo
            gThread(i) = 0
         end if
      end if
   Next i 

End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  InitListen
'  Starts all ip ports listening
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub InitListen()

   Dim i as Long
   
   For i = 1 To %MAX_IP_WAIT
      If gIPAddr(i) <> 0 Then
         gMainSock(i) = FreeFile
         Tcp Open Server Addr gIPAddr(i) Port gPort As gMainSock(i) TIMEOUT %TCP_TIMEOUT
         If Err Then
            MsgBox "Unable to listen to " + FormatIP$(gIPAddr(i)) + _
                   " Err=" + Format$(Err)
         Else
            Tcp Notify gMainSock(i), Accept To gDlgMain As %TCP_CONNECT            
         End If
      End If
   Next i

End Sub


'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  rsCommand
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function rsCommand (ByVal iCurrentSocket as Long, rIn as XFER_BUFFER_TYPE, _
                   sInBuffer as String) as Long

   Dim i as Long
   Dim iMoreLeft as Long
   Dim iLen as Long
   Dim sFile as String
   Dim sReturn as String
   Dim sBuffer as String
   Dim rOut As XFER_BUFFER_TYPE

   iMoreLeft = %True
   
   Select Case rIn.iCommand

      '-------- OPEN FILE FOR SHARED ACCESS
      Case %RSERVER_OPEN_FILE

         '- The length of the file name is the first
         '  two bytes (integer) after the structure.
         '  The file name follows it's length
         '
         sFile = Trim$(Mid$(sInBuffer, 3, Cvi(Mid$(sInBuffer, 1, 2))))
         LogEvent "OPEN " + sFile
         DisplayEvent "Open File " + sFile

         '- Open the file. If there's an error, the error
         '  is stored in the iResult member. Otherwise,
         '  the file handle is given to the iFile member.
         '
         EnterCritical
         On Error Resume Next
         rOut.iFile = FreeFile
         Open sFile For Binary Shared As #rOut.iFile
         If Err Then
            rOut.iFile = 0
         End If
         rOut.iResult = Err
         rOut.iDataLen = 0
         LeaveCritical

         '- Send back the results structure
         sReturn = Space$(Len(rOut))
         Lset sReturn = rOut
         Tcp Send iCurrentSocket, sReturn

      '-------- CLOSE FILE
      Case %RSERVER_CLOSE_FILE
         LogEvent "CLOSE FILE " + Format$(rIn.iFile)
         DisplayEvent "CLOSE " + Format$(rIn.iFile)
         Close #rIn.iFile
         rOut.iResult = 0
         rOut.iDataLen = 0
         rOut.iFile = 0
         sReturn = Space$(Len(rOut))
         Lset sReturn = rOut
         Tcp Send iCurrentSocket, sReturn

      '-------- GET DATA FROM FILE
      Case %RSERVER_GET
         DisplayEvent "GET " + Format$(rIn.iFile)

         '- How much data do they want?
         iLen = Cvl(Left$(sInBuffer, 4))

         '- Retrieve the data from the file
         On Error Resume Next
         sBuffer = Space$(iLen)
         Get #rIn.iFile,, sBuffer
         If Err Then
            sBuffer = ""
            rOut.iDataLen = 0
         Else
            rOut.iDataLen = Len(sBuffer)
         End If
         rOut.iResult = Err
         rOut.iFile = rIn.iFile

         '- Send back the response with the data
         sReturn = Space$(Len(rOut))
         Lset sReturn = rOut
         sReturn = sReturn + sBuffer

         Tcp Send iCurrentSocket, sReturn

      '-------- WRITE DATA TO FILE
      Case %RSERVER_PUT
         DisplayEvent "PUT " + Format$(rIn.iFile)
         On Error Resume Next
         Put #rIn.iFile,, sInBuffer
         rOut.iResult = Err
         rOut.iFile = rIn.iFile
         rOut.iDataLen = 0
         sReturn = Space$(Len(rOut))
         Lset sReturn = rOut
         Tcp Send iCurrentSocket, sReturn

      '-------- SEEK A POSITION IN THE FILE
      Case %RSERVER_SEEK
         DisplayEvent "SEEK " + Format$(rIn.iFile)
         On Error Resume Next
         Seek #rIn.iFile, Cvl(Left$(sInBuffer, 4))
         rOut.iFile = rIn.iFile
         rOut.iResult = Err
         rOut.iDataLen = 0
         sReturn = Space$(Len(rOut))
         Lset sReturn = rOut
         Tcp Send iCurrentSocket, sReturn

      '-------- GET LENGTH OF THE FILE
      Case %RSERVER_LOF
         DisplayEvent "LOF " + Format$(rIn.iFile)
         On Error Resume Next
         i = Lof(rIn.iFile)
         If Err Then
            rOut.iDataLen = 0
            sBuffer = ""
         Else
            rOut.iDataLen = 4
            sBuffer = Mkl$(i)
         End If
         rOut.iFile = rIn.iFile
         rOut.iResult = Err
         sReturn = Space$(Len(rOut))
         Lset sReturn = rOut
         sReturn = sReturn + sBuffer
         Tcp Send iCurrentSocket, sReturn

      '----------- No more to come
      Case %RSERVER_QUIT
         iMoreLeft = %False
         
      '----------- UNKNOWN instruction
      Case Else
         LogEvent "UNKNOWN INSTRUCTION:" + Str$(rIn.iCommand)
         DisplayEvent "Unknown Instruction:" + Str$(rIn.iCommand)
         rOut.iResult = 0
         rOut.iFile = 0
         rOut.iDataLen = 0
         sReturn = Space$(Len(rOut))
         Lset sReturn = rOut
         Tcp Send iCurrentSocket, sReturn

   End Select
   
   Function = iMoreLeft
   
End Function
                   
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  tcpHandleConnection
'  This function is called to create a new thread.  It sits in a loop 
'  responding to requests until the socket is closed (this generates an
'  error an the loop bails out. When it finishes, it closes the socket.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function tcpHandleConnection( ByVal iWhich as Long ) as Long

   Dim i as Long
   Dim iCurrentSocket as Long
   Dim iLen as Long
   Dim sFile as String
   Dim sResult as String
   Dim sBuffer as String
   Dim sInBuffer as String
   Dim rOut As XFER_BUFFER_TYPE
   Dim rIn as XFER_BUFFER_TYPE

   gThreadRunning(iWhich) = %True
   
   iCurrentSocket = gSock(iWhich)      
   Do
      
      '- If this fails, then there was a tcp error
      if tcpSafeReceive(iCurrentSocket, len(rIn), sBuffer) = %False then
         exit do
         
      '- Monitor the global shutdown flag
      elseif gShutdown then
         exit do
      
      '- Process the data
      else
         Lset rIn = sBuffer                     

         '- Get the variable portion of the buffer
         If rIn.iDataLen > 0 Then
            tcpSafeReceive iCurrentSocket, rIn.iDatalen, sInBuffer
         End If
      
         '- This returns %False if the client sent a disconnect                     
         if rsCommand(iCurrentSocket, rIn, sInBuffer) = %False then
            exit do
         end if
      end if
      Sleep 5
   Loop
   
   tcp close gSock(iWhich)
   gSock(iWhich) = %INVALID_SOCKET
   gThreadRunning(iWhich) = %False
   
   gActive = gActive - 1                  
   DisplayEvent "Disconnect: " + Format$(iWhich)

   Function = %True
   
End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  gDlgMain_WinProc
'  This is the event loop for the application
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Callback Function gDlgMain_WinProc

   Dim i As Long
   Dim iFound As Long
   Dim iCurrentSocket As Long
   Dim iCount As Long
   Dim iEmpty as Long
   Dim iMain as Long
   
   Select Case CbMsg

      Case %WM_INITDIALOG
         InitListen         
         
         '- Create a timer to handle thread cleanup
         SetTimer gDlgMain, %timer1, %CLEANUP_TIMER_INTERVAL, ByVal %NULL

      Case %WM_TIMER
         If cbwParam = %timer1 Then
            ThreadCleanup
         End If
         
      Case %TCP_CONNECT

         '- wparam contains the handle of the socket (FileAttr(hSock, 2) from accept event)
         Select Case LoWrd(CbLParam)

            '- Someone is attempting to connect to the server
            Case %FD_ACCEPT
            
               EnterCritical
               iFound = 0
               
               '- Find an empty connection
               iEmpty = 0
               For i = 1 To %MAX_CONNECTIONS
                  If gSock(i) = %INVALID_SOCKET Then
                     iEmpty = i
                     exit for
                  End If
               Next i
               
               '- Figure out which socket this is coming in on.
               iMain = 0
               For i = 1 To %MAX_IP_WAIT                  
                  If FileAttr(gMainSock(i), 2) = cbwparam Then
                     iMain = i
                  End iF
               Next i
               
               '- Now we know which socket the connect request is coming 
               '  in on and which one we should assign it to,
               '  so accept the connection and specify the notify events
               '
               if (iMain > 0) and (iEmpty > 0) then

                  '- Accept the connection
                  gSock(iEmpty) = FreeFile
                  Tcp Accept gMainSock(iMain) As gSock(iEmpty)
                  'Tcp Notify gSock(iEmpty), To gDlgMain As %TCP_CONNECT
                  DisplayEvent "FD_ACCEPT: " + Str$(gSock(iEmpty)) + " " + Str$(gMainSock(iMain))
                  LogEvent "CONNECT"
                  gActive = gActive + 1                  
                  
                  Thread Create tcpHandleConnection(iEmpty) To gThread(iEmpty)

               end if

               LeaveCritical
                              
         End Select
         Function = 1

      Case %WM_SYSCOMMAND
         If cbwParam = %SC_CLOSE Then
            KillTimer gDlgMain, %timer1
         End if
         
      Case %WM_DESTROY
         
         if gLog then
            Print #gLog, "----------------------------------------------"
            Print #gLog, "SHUT DOWN " + Date$ + "  " + Time$
            Print #gLog, "  Active Clients:" + Str$(gActive)
            Print #gLog, "      Total Hits:" + Str$(gHits)
            Print #gLog, "**********************************************"
         end if
         
   End Select

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  StopButton
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Callback Function btnStop_Click()

   Dialog End gDlgMain, 0
   Function = 1

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Initialize
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function Initialize() as Long
   
   Dim i as Long
   
   '- The configuration file is called rserver.ini
   '  and must be in the same directory as rserver.exe
   '   
   gIniFile = trim$(ExtractPath(GetExeName()))
   if right$(gIniFile, 1) <> "\" then
      gIniFile = gIniFile + "\"
   end if
   gIniFile = gIniFile + "rserver.ini"
   
   '- This is a multi-threaded application.
   '  I use a critical section to resolve thread conflicts
   '     
   InitializeCriticalSection gCS
      
   '- This program can be set to be multi-homed - that is, 
   '  listen on more than one IP address. The IP addresses
   '  are specified in the rserver.ini file required to be
   '  in the same directory as rserver.exe. If it is not
   '  there, then the default IP address for this computer
   '  is assumed. This is based on what is returned
   '  by Host ADDR "" To   
   '
   gIPCount = 0
   ReDim gIPAddr(%MAX_IP_WAIT) as Long
   For i = 1 to %MAX_IP_WAIT
      gIPAddr(i) = Int(val(GetIni(gIniFile, "server", "IPAddr" + Format$(i), "0")))
      if gIPAddr(i) <> 0 then
         incr gIPCount
      end if
   Next i

   Redim gMainSock(%MAX_IP_WAIT) as Long   
   
   '- Set the default if none were found in the .ini
   if gIPCount < 1 then
      gIPCount = 1
      Host ADDR "" To gIPAddr(1)
   end if
         
   '- The port is an .ini setting. There is a constant
   '  for the default.
   '
   gPort = Int(val(GetIni(gIniFile, "server", "port", Format$(%DEFAULT_PORT))))

   '- These are our connection place holders. Fill up the list with
   '  invalid connections.
   '
   ReDim gSock(1 To %MAX_CONNECTIONS) As Long
   For i = 1 To %MAX_CONNECTIONS
      gSock(i) = %INVALID_SOCKET
   Next i

   '- There will be on thread ID per connection
   Redim gThread(1 to %MAX_CONNECTIONS) as Long
   Redim gThreadRunning(1 to %MAX_CONNECTIONS) as Long
      
   gShutdown = %False

   '- Open the log file
   On Error Resume Next
   gLog = FreeFile
   Open "rserver.log" For Append As #gLog
   If Err Then
      MsgBox "Unable to create log file. Error =" + Str$(Err)
      gLog = 0
   Else
      Print #gLog, ""
      Print #gLog, "**********************************************"
      Print #gLog, "START UP " + Date$ + "  " +Time$
      Print #gLog, "----------------------------------------------"
   End If

   Function = %True   
End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  ShutDown
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub ShutDown()

   Dim i as Long
   Dim foo as Long
      
   '- All threads monitor this flag and
   '  will bail as soon as possible after
   '  seeing this.
   '
   gShutdown = %True
   Sleep 100
   
   '- Make sure to clean up any open sockets
   '  By closing a socket, the blocking tcp calls
   '  will stop blocking and error out. This
   '  allows the threads to finish executing.
   '
   For i = 1 To %MAX_CONNECTIONS
      If gSock(i) <> %INVALID_SOCKET Then
         Tcp Close gSock(i)       
         gSock(i) = %INVALID_SOCKET
      End If
   Next i

   '- Free any running threads
   For i = 1 to %MAX_CONNECTIONS
      if gThread(i) <> 0 then

         Thread Close gThread(i) to foo
         
         '- Wait for the thread to finish running
         '  When a thread finishes running, it sets its
         '  gThreadRunning() variable to be zero
         '
         Do until gThreadRunning(i) = %False
            Sleep 10
         Loop
         
      end if
   Next i

         
   '- Close any open files
   if gLog then 
      Close #gLog
      gLog = 0
   end if
   
   Close

   DeleteCriticalSection gCS

End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  gDlgMain_Create()
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub gDlgMain_Create()

   Dim i as Long
   Dim ipa as String
   
   '- Get a list of the IP addresses for display purposes
   For i = 1 to gIPCount
      if ipa = "" then
         ipa = FormatIP(gIPAddr(i))
      else
         ipa = ipa + "; " + FormatIP(gIPAddr(i))
      end if
   Next i 

   '- Create the main screen.
   Dialog New 0, "Binary Data Server on Port " + Format$(%DEFAULT_PORT),,, _
              200, 200, %WS_MINIMIZEBOX Or %WS_CAPTION Or %WS_SYSMENU, 0 To gDlgMain

   Control Add label, gDlgMain, -1, "Server IP(s)", 17, 10, 40, 8, %SS_RIGHT
   Control Add label, gDlgMain, -1, "Port", 17, 25, 40, 8, %SS_RIGHT
   Control Add label, gDlgMain, -1, "Started", 17, 40, 40, 8, %SS_RIGHT
   Control Add label, gDlgMain, -1, "Total Hits", 17, 55, 40, 8, %SS_RIGHT
   Control Add label, gDlgMain, -1, "Active Clients", 17, 70, 40, 8, %SS_RIGHT
   Control Add label, gDlgMain, -1, "Last Action", 17, 85, 40, 8, %SS_RIGHT

   Control Add label, gDlgMain, %labIP, ipa, 65, 10, 80, 8, %SS_LEFT
   Control Add label, gDlgMain, %labPort, Format$(gPort), 65, 25, 80, 8, %SS_LEFT
   Control Add label, gDlgMain, %labDate, Date$ + "   " + Time$, 65, 40, 80, 8, %SS_LEFT
   Control Add label, gDlgMain, %labHits, "0", 65, 55, 80, 8, %SS_LEFT
   Control Add label, gDlgMain, %labActive, "0", 65, 70, 80, 8, %SS_LEFT
   Control Add label, gDlgMain, %labAction, "", 65, 85, 135, 8, %SS_LEFT

   Control Add Button, gDlgMain, %IDCANCEL, "&Stop", 150, 177, 40, 15, 0 Call btnStop_Click
   
End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  PBMain
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function PbMain As Long

   Initialize  
   gDlgMain_Create
   Dialog Show Modal gDlgMain Call gDlgMain_WinProc
   Shutdown
   
End Function
